#!/usr/bin/perl -w
##******************************************************************************
##
## Example script demonstrating usage of Mentat::Handyman for generating
## IDEA messages.
##
## Author:      Jan Mach
## Last update: 2015-03-12
##
##******************************************************************************
##
## This script contains working example of usage of Handyman tool. Script is
## divided into following sections:
##  1. Loading the library
##  2. Instantination of Handyman object
##      A. Instantination using local configurations
##      B. Instantination using configurations from external JSON file
##  3. Library usage examples
##      A. Commandline like approach for generating messages
##          1. Generate the message into object in memory
##          2. Generate the message and store it directly into file in target dir
##      B. Hash like approach for generating messages
##          1. Generate the message into object in memory
##          2. Generate the message and store it directly into file in target dir
##
## More examples of how to use the libraries used in this script can be found in
## the appropriate test scripts or manual pages:
##
## Usefull manual pages:
##  man mentat-handyman
##  man Mentat::Handyman
##  man Mentat::Message::IDEA
##  man Mentat::Message::Builder::IDEA
##  man Mentat::Message::Validator::IDEA
##
##******************************************************************************

use strict;
use warnings;

##------------------------------------------------------------------------------
## 1. Loading the library
##------------------------------------------------------------------------------

## Setup the paths to the custom libraries, if necessary
## Note: this step is necessary, when the Mentat system is installed from tarball
## packages, otherwise the libraries should be installed on the path Perl looks
## into.
# use lib "/opt/libcesnet-toolkit-perl/lib";
# use lib "/opt/idea/lib_perl/lib";
# use lib "/opt/mentat/lib";

use Mentat::Handyman;

##------------------------------------------------------------------------------
## 2.A Instantinate the Handyman tool using local configurations
##
## Local configurations are the ones defined directly in script file. Following
## are the example configurations for both Debian package and tarball package
## installations of Mentat system. When experimenting, please (un)comment the
## appropriate section.
##------------------------------------------------------------------------------

## Prepare handyman configurations - example configurations for Debian package installation
my %handyman_cfg = (
    "template_file" =>        "/etc/mentat/msgtmplts.json",         # Template file from which to load the message templates [MANDATORY]
    "temp_folder" =>          "/var/mentat/spool/_handyman",        # Working directory  [MANDATORY]
    "pickup_folder" =>        "/var/mentat/spool/_pickup/incoming", # Target folder to which to store the generated messages [MANDATORY]
    "schema_file" =>          "/etc/mentat/schema-idea.json",       # JSON schema for message validation [MANDATORY]
    "template_schema_file" => "/etc/mentat/schema-msgtmplts.json",  # JSON schema message template validation [MANDATORY]
    "user" =>                 "mentat",                             # Name of the user, which will own the generated messages [MANDATORY]
    "message_type" =>         "idea",                               # Type of the generated message [MANDATORY]
    "strict" =>               1,                                    # Switch for strict error checking [OPTIONAL]
    "test_mode" =>            1,                                    # Switch for test mode [OPTIONAL]
    "builder_config" => {
            "name" => "cz.cesnet.node", # Name of the detection node node (reverse DNS name is suggested) [MANDATORY]
            "type" => "Honeypot, Test", # Type of the detection node (list of comma separated values) [OPTIONAL]
            "sw"   => "LaBrea",         # Name of the detection software [OPTIONAL]
            "note" => "Addtional note", # Additional note [OPTIONAL]
    }
);
## Prepare handyman configurations - example configurations for tarball installation
#my %handyman_cfg = (
#   "template_file" =>        "/opt/mentat/etc/msgtmplts.json",         # Template file from which to load the message templates [MANDATORY]
#   "temp_folder" =>          "/opt/mentat/var/spool/tmp",              # Working directory  [MANDATORY]
#   "pickup_folder" =>        "/opt/mentat/var/spool/_pickup/incoming", # Target folder to which to store the generated messages [MANDATORY]
#   "schema_file" =>          "/opt/mentat/etc/schema-idea.json",       # JSON schema for message validation [MANDATORY]
#   "template_schema_file" => "/opt/mentat/etc/schema-msgtmplts.json",  # JSON schema message template validation [MANDATORY]
#   "user" =>                 "mentat",                                 # Name of the user, which will own the generated messages [MANDATORY]
#   "message_type" =>         "idea",                                   # Type of the generated message [MANDATORY]
#   "strict" =>               1,                                        # Switch for strict error checking [OPTIONAL]
#   "test_mode" =>            1,                                        # Switch for test mode [OPTIONAL]
#   "builder_config" => {
#           "name" => "cz.cesnet.node", # Name of the detection node node (reverse DNS name is suggested) [MANDATORY]
#           "type" => "Honeypot, Test", # Type of the detection node (list of comma separated values) [OPTIONAL]
#           "sw"   => "LaBrea",         # Name of the detection software [OPTIONAL]
#           "note" => "Addtional note", # Additional note [OPTIONAL]
#   }
#);

## Instantinate the handyman object
#my $handyman = Mentat::Handyman->new(%handyman_cfg);

##------------------------------------------------------------------------------
## 2.B Instantinate the handyman tool using external configurations
##
## Handyman configurations can be pulled from external JSON file. This approach
## is particularly useful when Handyman tool is being used in the same way (with)
## the same configuration) in more than one place/script.
##------------------------------------------------------------------------------

## Instantinate Mentat::Handyman object using the configurations from external JSON
## file. When experimenting, please (un)comment the appropriate option.
my $handyman = Mentat::Handyman->load('./handyman-deb.json');
#my $handyman = Mentat::Handyman->load('./handyman-tar.json');

##------------------------------------------------------------------------------
## 3. Use the Handyman tool to generate the IDEA messages
##
## Following is only example of general algorithm for processing arbitrary events.
##------------------------------------------------------------------------------

my ($template_id, $detect_time, $rv, $filename, @args_a, %args_h, $message);
my @events = (1,2,3);
foreach my $event (@events) {
    eval {
        ##----------------------------------------------------------------------
        ## When processing the events, somehow choose the identifier of appropriate
        ## template from the template file and set/prepare the data, that will be
        ## filled into the message.
        ##----------------------------------------------------------------------
        ## Identifier of the template to be used
        $template_id = 'sserv-001';
        ## Detection time - ussually supplied as commandline option
        $detect_time = 1356955200 || time;
        ##----------------------------------------------------------------------
        ## Option 3.A - Commandline like approach
        ##              List of commandline arguments to be filled into the message template
        ## Note: The 3.A approach is the legacy coming from command line usage
        @args_a = ('192.0.2.3','6667','1','192.0.2.3 6667 (#crack-me) C');
        ##----------------------------------------------------------------------
        ## Option 3.B - Hash like approach
        ##              Variables are given in a hash right away
        ## Note: Names of the keys in the hash depend on the appropriate template
        %args_h = ('detect_time' => $detect_time,
                   '1' => '192.0.2.3',
                   '2' => '6667',
                   '3' => '1',
                   '4' => '192.0.2.3 6667 (#crack-me) C');

        ##----------------------------------------------------------------------
        ## Option 3.A.1 - Generate the message to the object - commandline like approach
        #$message = $handyman->generate($template_id, {'detect_time' => $detect_time}, @args);
        ##----------------------------------------------------------------------
        ## Option 3.B.1 - Generate the message to the object - hash like approach
        $message = $handyman->generate($template_id, \%args_h);
        ## Optional example - Add additional message attachment
        $message->path_node_set('Attach[2]/ContentType', 'text/plain');
        $message->path_node_set('Attach[2]/Note', 'Additional info');
        $message->path_node_set('Attach[2]/Content', 'Some very necessary additional information');
        ## For now just display the message to STDOUT
        print $message->to_string(1);

        ##----------------------------------------------------------------------
        ## Option 3.A.2 - Generate the message directly to some file in target directory - command line like approach
        #($rv, $filename) = $handyman->generate_to_file($template_id, {'detect_time' => $detect_time}, @args_a);
        ##----------------------------------------------------------------------
        ## Option 3.B.2 - Generate the message directly to some file in target directory - hashed approach
        #($rv, $filename) = $handyman->generate_to_file($template_id, \%args_h);
        #print "Generated and stored message to file '$filename'\n";
    };
    ## Handle errors here and continue processing
    if ($@) {
        ## SOMEHOW DEAL WITH ERRORS HERE
        print STDERR "Event $event could not be processed: $@\n";
    }
}
